/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.actions;

import org.eclipse.cosmos.rm.smlif.SMLPlugin;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

/**
 * An abstract class that all other object action delegates are expected
 * to extend.
 * 
 * @author Ali Mehregani
 */
public abstract class AbstractAction implements IObjectActionDelegate
{
	/**
	 * The current selection
	 */
	private IStructuredSelection selection;
	
	
	/** 
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart)
	{
	}

	
	/**
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection)
	{
		this.selection = selection instanceof IStructuredSelection ? 
				(IStructuredSelection)selection : null;			
	}


	/**
	 * @return the selection
	 */
	public IStructuredSelection getSelection()
	{
		return selection;
	}
	
	protected boolean noValidSelection()
	{
		if (selection == null || selection.isEmpty())
		{
			MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
					SMLMessages.errorNoSelectionTitle, SMLMessages.errorNoSelection);
			return true;
		}
		return false;
		
	}

}
