/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.editor;


import org.eclipse.cosmos.rm.smlif.SMLPlugin;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

/**
 * The overview page of the SML-IF editor
 * 
 * @author Ali Mehregani
 */
public class OverviewPage extends AbstractFormPage
{
	/**
	 * The id of this page
	 */
	public static final String ID = SMLPlugin.PLUGIN_ID + ".OverviewPage";
	
	/**
	 * Constructor
	 * 
	 * @param editor The form editor
	 */
	public OverviewPage(FormEditor editor)
	{
		super(editor, ID, SMLMessages.editorOverviewTitle);
	}
	
	
	/**
	 * @see org.eclipse.cosmos.rm.smlif.internal.editor.AbstractFormPage#createLeftColumn(org.eclipse.swt.widgets.Composite)
	 */
	protected void createLeftColumn(Composite parent)
	{
		FormToolkit toolkit = getManagedForm().getToolkit();
		Section section = createSection(parent);
		
		Composite sectionClient = toolkit.createComposite(section);
		sectionClient.setLayout(new GridLayout(2, false));
		sectionClient.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
				
		SMLIFEditor editor = (SMLIFEditor)getEditor();
		
		createTextField(sectionClient, SMLMessages.editorName, editor.retrieveField(SMLIFEditor.TYPE_IDENTITY, ISMLConstants.NAME_ELEMENT), this);
		createTextField(sectionClient, SMLMessages.editorDisplayName, editor.retrieveField(SMLIFEditor.TYPE_IDENTITY, ISMLConstants.DISPLAY_NAME_ELEMENT), this);
		createTextField(sectionClient, SMLMessages.editorBaseURI, editor.retrieveField(SMLIFEditor.TYPE_IDENTITY, ISMLConstants.BASE_URI_ELEMENT), this);
		createTextField(sectionClient, SMLMessages.editorDescription, true, editor.retrieveField(SMLIFEditor.TYPE_IDENTITY, ISMLConstants.DESCRIPTION_ELEMENT), this);
		
		makeSectionReady (section, sectionClient, SMLMessages.editorGeneralInformation);
	}



	/**
	 * @see org.eclipse.cosmos.rm.smlif.internal.editor.AbstractFormPage#createRightColumn(org.eclipse.swt.widgets.Composite)
	 */
	protected void createRightColumn(Composite parent)
	{
		FormToolkit toolkit = getManagedForm().getToolkit();
		Section section = createSection(parent);
		
		Composite sectionClient = toolkit.createComposite(section);
		GridLayout gl = new GridLayout();
		gl.marginTop = 50;
		sectionClient.setLayout(gl);
		sectionClient.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
				
		toolkit.createHyperlink(sectionClient, SMLMessages.editorValidateDocument, SWT.WRAP);				
		makeSectionReady (section, sectionClient, SMLMessages.editorGeneralInformation);
	}

}
