package org.eclipse.cosmos.rm.validation.internal.core;

import java.util.List;
import java.util.Map;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DocumentDOMBuilder;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Generated code for the test suite <b>TestDatastructureBuilderImpl</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestDatastructureBuilderImpl.testsuite</i>.
 */
public class TestDatastructureBuilderImpl extends HyadesTestCase
{
	/**
	 * Constructor for TestDatastructureBuilderImpl.
	 * @param name
	 */
	public TestDatastructureBuilderImpl(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestDatastructureBuilderImpl</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testDatastructureBuilderImpl = new HyadesTestSuite("TestDatastructureBuilderImpl");
		testDatastructureBuilderImpl.setArbiter(DefaultTestArbiter.INSTANCE).setId("D14389B7B6BCE5E3758AD7C09B3F11DB");
		testDatastructureBuilderImpl.addTest(new TestDatastructureBuilderImpl("testRemoteDocuments").setId("D14389B7B6BCE5E37735A1909B3F11DB").setTestInvocationId("D14389B7B6BCE5E37E3B44E09B3F11DB"));
		return testDatastructureBuilderImpl;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	
	/**
	 * testRemoteDocuments
	 * @throws Exception
	 */
	public void testRemoteDocuments() throws Exception
	{		
		DataBuilderRegistry.instance().registerDataStructureBuilder(DocumentDOMBuilder.ID, new DocumentDOMBuilder());
		UtilityTester.parseDocument(this.getClass().getClassLoader().getResourceAsStream("test-resources/others/remote-document.xml"), new DatastructureBuilderImpl());
		Map domDocuments = (Map)SMLCommonUtil.retrieveDataStructure(DocumentDOMBuilder.ID);
		List list = (List)domDocuments.get(DocumentDOMBuilder.NO_ALIAS);
		assertEquals("The size of the list must be 2", 2, list.size());
		assertEquals("The id of the first element must be 1000", 1000, retrieveId((Node)list.get(0)));
		assertEquals("The id of the second element must be 1001", 1001, retrieveId((Node)list.get(1)));
	}

	private int retrieveId(Node node)
	{
		NodeList children = node.getChildNodes();
		Node idNode = null;
		for (int i = 0, childCount = children.getLength(); i < childCount; i++)
		{
			Node child = children.item(i);
			if ("ID".equals(child.getLocalName()))
			{
				idNode = child;
				break;
			}
		}
		
		if (idNode == null)
			return -1;		
		
		children = idNode.getChildNodes();
		Node id = children == null || children.getLength() != 1 ? null : children.item(0);
		return id == null || id.getNodeType() != Node.TEXT_NODE ? -1 : Integer.parseInt(id.getNodeValue());
	}
}
