package org.eclipse.cosmos.rm.validation.internal.core;

import java.util.Hashtable;
import java.util.Map;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.FileOutput;
import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DocumentDOMBuilder;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestPluginDatastructureBuilder</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/plugin-junits/TestPluginDatastructureBuilder.testsuite</i>.
 */
public class TestPluginDatastructureBuilder extends HyadesTestCase
{
	/**
	 * Constructor for TestPluginDatastructureBuilder.
	 * @param name
	 */
	public TestPluginDatastructureBuilder(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestPluginDatastructureBuilder</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testPluginDatastructureBuilder = new HyadesTestSuite("TestPluginDatastructureBuilder");
		testPluginDatastructureBuilder.setArbiter(DefaultTestArbiter.INSTANCE).setId("C205F1E79253A3D113D7BDA09CD311DB");
		testPluginDatastructureBuilder.addTest(new TestPluginDatastructureBuilder("testModelUnitFiles").setId("C205F1E79253A3D118FBCFB09CD311DB").setTestInvocationId("C205F1E79253A3D13B419D709CD311DB"));
		testPluginDatastructureBuilder.addTest(new TestPluginDatastructureBuilder("testModelUnitFolder").setId("C205F1E79253A3D119139D709CD311DB").setTestInvocationId("C205F1E79253A3D13B48A2509CD311DB"));
		testPluginDatastructureBuilder.addTest(new TestPluginDatastructureBuilder("testModelUnitMixture").setId("C205F1E79253A3D1193297209CD311DB").setTestInvocationId("C205F1E79253A3D13B4D84509CD311DB"));
		return testPluginDatastructureBuilder;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	private void parseUnits(String[] filePaths, String[] aliases)
	{
		DataBuilderRegistry.instance().registerDataStructureBuilder(DocumentDOMBuilder.ID, new DocumentDOMBuilder());
		DatastructureBuilderImpl datastructureBuilder = new DatastructureBuilderImpl();
		Map attributes = new Hashtable();
		
		String files = "";
		for (int i = 0; i < filePaths.length; i++)
		{
			if (i > 0)
				files += ",";
			files += filePaths[i];
		}
		FileOutput fileOutput = new FileOutput();		
		attributes.put(IValidator.ATTRIBUTE_INSTANCE, files);
		attributes.put(IValidator.ATTRIBUTE_INPUT_TYPE, IValidator.VALUE_SML_UNIT);
		attributes.put(IValidator.ATTRIBUTE_OUTPUT, fileOutput);
		attributes.put(IValidator.ATTRIBUTE_FILE_OUTPUT, "D://temp//validator.output");
		datastructureBuilder.initialize(attributes);
		datastructureBuilder.validate();
		
		Map domDocuments = (Map)SMLCommonUtil.retrieveDataStructure(DocumentDOMBuilder.ID);
		assertNotNull("The DOM document map is null", domDocuments);
		assertEquals("The size of the DOM document map is expected to be " + aliases.length + 1, aliases.length + 1, domDocuments.size());
		for (int i = 0; i < aliases.length; i++)
		{
			assertNotNull("The alias " + aliases[i] + " does not exist", domDocuments.get(aliases[i]));
		}
	}

	
	/**
	 * testModelUnitFiles This test requires the context workbench to contain the DataCenterSample project
	 * @throws Exception
	 */
	public void testModelUnitFiles() throws Exception
	{
		parseUnits (new String[]{
				"/DataCenterSample/abstract/compositeApplication.xml",
				"/DataCenterSample/abstract/compositeApplication.xsd"
			},
			new String[]{
				"/DataCenterSample/abstract/compositeApplication.xml"
			});
	}

	/**
	 * testModelUnitFolder This test requires the context workbench to contain the DataCenterSample project
	 * @throws Exception
	 */
	public void testModelUnitFolder() throws Exception
	{
		parseUnits (new String[]{
				"/DataCenterSample/abstract"
			},
			new String[]{
				"/DataCenterSample/abstract/compositeApplication.xml",
				"/DataCenterSample/abstract/configurationView.xml",
				"/DataCenterSample/abstract/inventoryView.xml",
				"/DataCenterSample/abstract/network.xml"
			});
	}

	/**
	 * testModelUnitMixture This test requires the context workbench to contain the DataCenterSample project
	 * @throws Exception
	 */
	public void testModelUnitMixture() throws Exception
	{
		parseUnits (new String[]{
				"/DataCenterSample/abstract",
				"/DataCenterSample/os/unix",
				"/DataCenterSample/os/windows/osWindows.xsd",
				"/DataCenterSample/os/windows/WindowsServer1.xml"
			},
			new String[]{
				"/DataCenterSample/abstract/compositeApplication.xml",
				"/DataCenterSample/abstract/configurationView.xml",
				"/DataCenterSample/abstract/inventoryView.xml",
				"/DataCenterSample/abstract/network.xml",
				"/DataCenterSample/os/unix/linux/Redhat1.xml",
				"/DataCenterSample/os/unix/linux/Redhat2.xml",
				"/DataCenterSample/os/windows/WindowsServer1.xml"
			});
	}

}
