package org.eclipse.cosmos.rm.validation.internal.reference;

import java.io.IOException;
import java.io.StringReader;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.cosmos.rm.validation.internal.databuilders.AbstractDataBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DocumentDOMBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.NamespaceContextBuilder;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Generated code for the test suite <b>TestDrefXPathFunction</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestDrefXPathFunction.testsuite</i>.
 */
public class TestDrefXPathFunction extends HyadesTestCase
{
	private final String SINGLE_STUDENT = 				
		"<Student xmlns=\"urn:university\" SSN=\"123-22-4444\">\n" +
		"<ID>1000</ID>\n" +
		"<Name>John</Name>\n" +
		"</Student>";
	
	private Node context;
	
	/**
	 * Constructor for TestDrefXPathFunction.
	 * @param name
	 */
	public TestDrefXPathFunction(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestDrefXPathFunction</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testDrefXPathFunction = new HyadesTestSuite("TestDrefXPathFunction");
		testDrefXPathFunction.setArbiter(DefaultTestArbiter.INSTANCE).setId("F047EAA18640A2EDB3FCDE808BA511DB");
		testDrefXPathFunction.addTest(new TestDrefXPathFunction("testSingle").setId("F047EAA18640A2EDBAC9C9208BA611DB").setTestInvocationId("F047EAA18640A2EDE9BEF7008BA611DB"));
		testDrefXPathFunction.addTest(new TestDrefXPathFunction("testMultiple").setId("F047EAA18640A2EDBB51BDD08BA611DB").setTestInvocationId("F047EAA18640A2EDE9C3B1F08BA611DB"));
		testDrefXPathFunction.addTest(new TestDrefXPathFunction("testMultipleNonRoot").setId("F047EAA18640A2ED859E7CF08BA611DB").setTestInvocationId("F047EAA18640A2EDE9B7CB108BA611DB"));
		return testDrefXPathFunction;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	private void invokeDref(final int EXPECTED_LENGTH, final String EXPECTED_NODE_NAME) throws XPathExpressionException
	{
		XPathExpression xpathExp = SMLCommonUtil.xpath.compile("smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)");
		
		NodeList nodeList = (NodeList)xpathExp.evaluate(context, XPathConstants.NODESET);
		assertNotNull("The result is expected to be a node list", nodeList);
		assertEquals("The size of the node list is expected to be " + EXPECTED_LENGTH, EXPECTED_LENGTH, nodeList.getLength());
		for (int i = 0; i < EXPECTED_LENGTH; i++)
		{
			Node node = nodeList.item(i);
			assertEquals("The name of the referenced node is expected to be " + EXPECTED_NODE_NAME, EXPECTED_NODE_NAME, node.getNodeName());
		}	
	}

	/**
	 * testSingle 
	 * Verify that deref() can resolve a single reference.   The deref() function must return a 
	 * single node (i.e. a node set with a single node) that corresponds to the element node of 
	 * the reference’s target when thereference targets a single element in the model.
	 * @throws Exception
	 */
	public void testSingle() throws Exception
	{
		initialize("University-Single.xml", 
				new String[]{"Student1"}, 
				createStudentNode(SINGLE_STUDENT, false));
		invokeDref(1, "Student");	
	}

	/**
	 * testMultiple
	 * Verify that deref() can accept a node set of reference elements, i.e., elements for which 
	 * sml:ref=”true”, and return a node set that is union of the element nodes targeted by 
	 * references in the node set such that the reference targets exactly one element in the model. The returned node set must not contain any nodes for references that do not target an element in the model.
	 * @throws Exception
	 */
	public void testMultiple() throws Exception
	{
		initialize("University-Multiple.xml", 
				new String[]{"Student1", "Student2", "Student3", "Student4"}, 
				createStudentNode(SINGLE_STUDENT, false));
		invokeDref(4, "Student");
	}

	
	/**
	 * testMultipleNonRoot
	 * Verify that deref() can resolve references that target non-root 
	 * elements using XPointer scheme.
	 * @throws Exception
	 */
	public void testMultipleNonRoot() throws Exception
	{
		initialize("University-MultipleNonRoot.xml", new String[]{"Students.xml"}, createStudentNode("Students-MultipleNonRoot.xml", true));
		DataBuilderRegistry.instance().registerDataStructureBuilder(NamespaceContextBuilder.ID, new NamespaceContextBuilder());
		invokeDref(3, "Student");
	}	
	
	private void initialize(String fileName, String[] aliases, Node aliasDoc) throws SAXException, IOException, ParserConfigurationException
	{		
		context = UtilityTester.domParseDocument(this.getClass().getClassLoader().getResourceAsStream("test-resources/deref/" + fileName)).getChildNodes().item(0);
		NamespaceContext namespaceContext = new NamespaceContext(){

			public String getNamespaceURI(String arg0)
			{
				if ("tns".equals(arg0))
					return "urn:university";
				else if ("smlfn".equals(arg0))
					return "http://schemas.serviceml.org/sml/function/2006/07";
				return "";
			}

			public String getPrefix(String arg0)
			{
				return null;
			}

			public Iterator getPrefixes(String arg0)
			{
				return null;
			}};
			
		SMLCommonUtil.xpath.setNamespaceContext(namespaceContext);
		UtilityTester.registerIdentityStructure(null, null, "http://interop.serviceml.org");
		DataBuilderRegistry registry = DataBuilderRegistry.instance();	

		UtilityTester.registerReftypeElements(new String[][] {
					new String[]{"urn:university", "EnrolledStudent"}
				});
	
		final Map domDocuments = new Hashtable();
		for (int i = 0; i < aliases.length; i++)
		{
			domDocuments.put(aliases[i], aliasDoc);
		}		
		registry.registerDataStructureBuilder(DocumentDOMBuilder.ID, new AbstractDataBuilder(){

			public Object getDataStructure()
			{
				return domDocuments;
			}});
	}
	
	private Node createStudentNode(String document, boolean isFile) throws SAXException, IOException, ParserConfigurationException
	{
		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
        docBuilderFactory.setNamespaceAware(true);
        docBuilderFactory.setIgnoringElementContentWhitespace(true);
        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();        
		Document doc = docBuilder.parse(isFile ? 
				  new InputSource(this.getClass().getClassLoader().getResourceAsStream("test-resources/deref/Students-MultipleNonRoot.xml"))
				: new InputSource(new StringReader(document)));
		return doc.getChildNodes().item(0);
	}
}
