/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.NamespaceContext;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class NamespaceContextBuilder extends AbstractDataBuilder
{
	/**
	 * The ID of this data builder
	 */
	public static final String ID = SMLActivator.PLUGIN_ID + ".NamespaceContextBuilder";
	
	/**
	 * The mapped namespace
	 */
	private MappedNamespaceContext mappedNamespace;
	
	/**
	 * Constructor
	 */
	public NamespaceContextBuilder()
	{
		mappedNamespace = new MappedNamespaceContext();
	}
	
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		/* Does not need to be implemented */ 
	}
	
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		/* Does not need to be implemented */
	}

	public Object getDataStructure()
	{		
		return mappedNamespace;
	}
	
	public static class MappedNamespaceContext implements NamespaceContext
	{
		/**
		 * The URI, prefix relation is stored in a map
		 * KEY = URI
		 * VALUE = A list of prefix
		 */
		private Map contextWithURIKey;
		
		/**
		 * The URI, prefix relation is stored in a map
		 * KEY = Prefix
		 * VALUE = The URI
		 */
		private Map contextWithPrefixKey;

		public String getNamespaceURI(String prefix)
		{
			if (contextWithPrefixKey == null)
				return null;
			String uri = (String)contextWithPrefixKey.get(prefix);
			return uri == null ? IValidationConstants.EMPTY_STRING : uri;
		}

		public String getPrefix(String uri)
		{
			return (String)getPrefixGeneric(uri, true);
		}

		public Iterator getPrefixes(String uri)
		{
			return (Iterator)getPrefixGeneric(uri, false);
		}
		
		private Object getPrefixGeneric (String uri, boolean first)
		{
			if (contextWithURIKey == null)
				return null;
			List prefixes = (List) contextWithURIKey.get(uri);
			if (prefixes != null && prefixes.size() > 0)
				return first ? prefixes.get(0) : prefixes.iterator();
			return null;
		}
		
		public void addEntry (String prefix, String uri)
		{
			if (contextWithURIKey == null)
				contextWithURIKey = new Hashtable();
			if (contextWithPrefixKey == null)
				contextWithPrefixKey = new Hashtable();
			
			List listOfPrefix = (List)contextWithURIKey.get(uri);
			if (listOfPrefix == null)
			{
				listOfPrefix = new ArrayList();
				contextWithURIKey.put(uri, listOfPrefix);
			}
			listOfPrefix.add(prefix);
			contextWithPrefixKey.put(prefix, uri);
		}
		
	}
}
